<?php
/**
 * A mcrypt-based encryption stream
 *
 * @author		Roland Haeder <webmaster@ship-simu.org>
 * @version		0.0.0
 * @copyright	Copyright (c) 2007, 2008 Roland Haeder, 2009 - 2012 Core Developer Team
 * @license		GNU GPL 3.0 or any newer version
 * @link		http://www.ship-simu.org
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
class McryptStream extends BaseStream implements EncryptableStream {
	/**
	 * Separator on many places
	 */
	const DATA_PAYLOAD_SEPARATOR = '|';

	/**
	 * Protected constructor
	 *
	 * @return	void
	 */
	protected function __construct () {
		// Call parent constructor
		parent::__construct(__CLASS__);
	}

	/**
	 * Creates an instance of this node class
	 *
	 * @param	$rngInstance		An RNG instance
	 * @return	$streamInstance		An instance of this node class
	 */
	public static final function createMcryptStream (RandomNumberGenerator $rngInstance) {
		// Get a new instance
		$streamInstance = new McryptStream();

		// Set the RNG instance
		$streamInstance->setRngInstance($rngInstance);

		// Return the instance
		return $streamInstance;
	}

	/**
	 * Encrypt the string with fixed salt
	 *
	 * @param	$str		The unencrypted string
	 * @param	$key		Optional key, if none provided, a random key will be generated
	 * @return	$encrypted	Encrypted string
	 */
	public function encryptStream ($str, $key = NULL) {
		// Init crypto module
		$iv_size = mcrypt_get_iv_size(MCRYPT_RIJNDAEL_256, MCRYPT_MODE_ECB);
		$iv = mcrypt_create_iv($iv_size, MCRYPT_RAND);

		// Generate key, if none provided
		if (is_null($key)) {
			// None provided
			$key = $this->getRngInstance()->generateKey();
		} // END - if

		// Add some "payload" to the string
		switch ($this->getRngInstance()->randomNumber(0, 8)) {
			case 0:
				$payloadString = crc32($this->getRngInstance()->randomString(10)) . self::DATA_PAYLOAD_SEPARATOR . base64_encode($str) . self::DATA_PAYLOAD_SEPARATOR . crc32($this->getRngInstance()->randomString(20));
				break;

			case 1:
				$payloadString = crc32($this->getRngInstance()->randomString(10)) . self::DATA_PAYLOAD_SEPARATOR . base64_encode($str) . self::DATA_PAYLOAD_SEPARATOR . md5($this->getRngInstance()->randomString(20));
				break;

			case 2:
				$payloadString = crc32($this->getRngInstance()->randomString(10)) . self::DATA_PAYLOAD_SEPARATOR . base64_encode($str) . self::DATA_PAYLOAD_SEPARATOR . sha1($this->getRngInstance()->randomString(20));
				break;

			case 3:
				$payloadString = md5($this->getRngInstance()->randomString(10)) . self::DATA_PAYLOAD_SEPARATOR . base64_encode($str) . self::DATA_PAYLOAD_SEPARATOR . crc32($this->getRngInstance()->randomString(20));
				break;

			case 4:
				$payloadString = md5($this->getRngInstance()->randomString(10)) . self::DATA_PAYLOAD_SEPARATOR . base64_encode($str) . self::DATA_PAYLOAD_SEPARATOR . md5($this->getRngInstance()->randomString(20));
				break;

			case 5:
				$payloadString = md5($this->getRngInstance()->randomString(10)) . self::DATA_PAYLOAD_SEPARATOR . base64_encode($str) . self::DATA_PAYLOAD_SEPARATOR . sha1($this->getRngInstance()->randomString(20));
				break;

			case 6:
				$payloadString = sha1($this->getRngInstance()->randomString(10)) . self::DATA_PAYLOAD_SEPARATOR . base64_encode($str) . self::DATA_PAYLOAD_SEPARATOR . crc32($this->getRngInstance()->randomString(20));
				break;

			case 7:
				$payloadString = sha1($this->getRngInstance()->randomString(10)) . self::DATA_PAYLOAD_SEPARATOR . base64_encode($str) . self::DATA_PAYLOAD_SEPARATOR . md5($this->getRngInstance()->randomString(20));
				break;

			case 8:
				$payloadString = sha1($this->getRngInstance()->randomString(10)) . self::DATA_PAYLOAD_SEPARATOR . base64_encode($str) . self::DATA_PAYLOAD_SEPARATOR . sha1($this->getRngInstance()->randomString(20));
				break;
		}

		// Encrypt the string
		$encrypted = mcrypt_encrypt(MCRYPT_RIJNDAEL_256, $key, $payloadString, MCRYPT_MODE_ECB, $iv);

		// Return the string
		return $encrypted;
	}

	/**
	 * Decrypt the string with fixed salt
	 *
	 * @param	$encrypted	Encrypted string
	 * @param	$key		Optional key, if none provided, a random key will be generated
	 * @return	$str		The unencrypted string
	 */
	public function decryptStream ($encrypted, $key = NULL) {
		// Init crypto module
		$iv_size = mcrypt_get_iv_size(MCRYPT_RIJNDAEL_256, MCRYPT_MODE_ECB);
		$iv = mcrypt_create_iv($iv_size, MCRYPT_RAND);

		// Shall we use a default key or custom?
		if (is_null($key)) {
			// Generate (default) key
			$key = $this->getRngInstance()->generateKey();
		} // END - if

		// Decrypt the string
		$payloadString = mcrypt_decrypt(MCRYPT_RIJNDAEL_256, $key, $encrypted, MCRYPT_MODE_ECB, $iv);

		// Get the real string out
		$strArray = explode(self::DATA_PAYLOAD_SEPARATOR, $payloadString);

		// Does the element count match?
		assert(count($strArray) == 3);

		// Decode the string
		$str = base64_decode($strArray[1]);

		// Trim trailing nulls away
		$str = rtrim($str, "\0");

		// Return the string
		return $str;
	}

	/**
	 * Streams the data and maybe does something to it
	 *
	 * @param	$data	The data (string mostly) to "stream"
	 * @return	$data	The data (string mostly) to "stream"
	 * @throws	UnsupportedOperationException	If this method is called (which is a mistake)
	 */
	public function streamData ($data) {
		$this->debugOutput('Unhandled ' . strlen($data) . ' bytes in this stream.');
		throw new UnsupportedOperationException(array($this, __FUNCTION__), self::EXCEPTION_UNSPPORTED_OPERATION);
	}
}

// [EOF]
?>
